@@ -19,8 +19,10 @@
 #'          \code{\link{UChp}}
 #'          
 #' @examples
-#' m1 <- UC(log(sales))
+#' \dontrun{
+#' m1 <- UC(log(AirPassengers))
 #' m1 <- UCcomponents(m1)
+#' }
 #' @rdname UCcomponents
 #' @export
 UCcomponents= function(sys){
@@ -35,7 +37,8 @@ UCcomponents= function(sys){
         u = sys$u
     }
     rubbish = c(sys$hidden$d_t, sys$hidden$innVariance, sys$hidden$objFunValue, TRUE, 
-                sys$outlier, sys$arma, sys$iter, sys$hidden$seas)
+                sys$outlier, sys$arma, sys$iter, sys$hidden$seas, sys$lambda,
+                sys$hidden$MSOE, sys$hidden$PTSnames)
     rubbish2 = cbind(sys$grad, sys$hidden$constPar, sys$hidden$typePar)
     rubbish3 = cbind(sys$hidden$ns, sys$hidden$nPar)
     output = UCompC("components", y, u, sys$model, sys$periods, sys$rhos,
@@ -43,7 +46,8 @@ UCcomponents= function(sys){
                     sys$stepwise, sys$hidden$estimOk, sys$p0, sys$v, sys$yFitV,
                     sys$hidden$nonStationaryTerms, rubbish3, sys$hidden$harmonics,
                     as.vector(sys$criteria), sys$hidden$cycleLimits, 
-                    cbind(sys$hidden$beta, sys$hidden$betaV), sys$hidden$typeOutliers)
+                    cbind(sys$hidden$beta, sys$hidden$betaV), sys$hidden$typeOutliers,
+                    sys$TVP, sys$trendOptions, sys$seasonalOptions, sys$irregularOptions)
     # Convert to R list
     sys$comp = output$comp
     sys$compV = output$compV
@@ -63,51 +67,51 @@ UCcomponents= function(sys){
         sys$comp = t(matrix(sys$comp, m, n))
         sys$compV = t(matrix(sys$compV, m, n))
     }
-    namesComp = c("Level", "Slope", "Seasonal", "Irregular")
-    if (nCycles > 0){
-        for (i in 1 : nCycles){
-            namesComp = c(namesComp, paste0("Cycle", i))
-        }
-    }
-    # Inputs names
-    if (k > 0){
-        nOut = 0;
-        if (sys$hidden$typeOutliers[1, 2] != -1){
-            nOut = dim(sys$hidden$typeOutliers)[1]
-        }
-        nU = k - nOut
-        if (nU > 0){
-            for (i in 1 : nU){
-                namesComp = c(namesComp, paste0("Exogenous", i))
-            }
-        }
-        if (nOut > 0){
-            for (i in 1 : nOut){
-                namei = "AO"
-                if (sys$hidden$typeOutliers[i, 1] == 1){
-                    namei = "LS"
-                } else if (sys$hidden$typeOutliers[i, 1] == 2){
-                    namei = "SC"
-                }
-                namesComp = c(namesComp, paste0(namei, sys$hidden$typeOutliers[i, 2]))
-            }
-        }
-    }
-    # Eliminating components that are zero
-    n = dim(sys$comp)[1] - sys$h
-    ind = NULL
-    for (i in 1 : 3){
-        if (max(sys$comp[1 : n, i], na.rm = TRUE) == 0){
-            ind = c(ind, i)
-        }
-    }
-    if (max(abs(sys$comp[1 : n, 4]), na.rm = TRUE) < 1e-12)
-        ind = c(ind, 4)
-    if (length(ind) > 0){
-        sys$comp = sys$comp[, -ind]
-        sys$compV = sys$compV[, -ind]
-        namesComp = namesComp[-ind]
-    }
+    # namesComp = c("Level", "Slope", "Seasonal", "Irregular")
+    # if (nCycles > 0){
+    #     for (i in 1 : nCycles){
+    #         namesComp = c(namesComp, paste0("Cycle", i))
+    #     }
+    # }
+    # # Inputs names
+    # if (k > 0){
+    #     nOut = 0;
+    #     if (sys$hidden$typeOutliers[1, 2] != -1){
+    #         nOut = dim(sys$hidden$typeOutliers)[1]
+    #     }
+    #     nU = k - nOut
+    #     if (nU > 0){
+    #         for (i in 1 : nU){
+    #             namesComp = c(namesComp, paste0("Exogenous", i))
+    #         }
+    #     }
+    #     if (nOut > 0){
+    #         for (i in 1 : nOut){
+    #             namei = "AO"
+    #             if (sys$hidden$typeOutliers[i, 1] == 1){
+    #                 namei = "LS"
+    #             } else if (sys$hidden$typeOutliers[i, 1] == 2){
+    #                 namei = "SC"
+    #             }
+    #             namesComp = c(namesComp, paste0(namei, sys$hidden$typeOutliers[i, 2]))
+    #         }
+    #     }
+    # }
+    # # Eliminating components that are zero
+    # n = dim(sys$comp)[1] - sys$h
+    # ind = NULL
+    # for (i in 1 : 3){
+    #     if (max(sys$comp[1 : n, i], na.rm = TRUE) == 0){
+    #         ind = c(ind, i)
+    #     }
+    # }
+    # if (max(abs(sys$comp[1 : n, 4]), na.rm = TRUE) < 1e-12)
+    #     ind = c(ind, 4)
+    # if (length(ind) > 0){
+    #     sys$comp = sys$comp[, -ind]
+    #     sys$compV = sys$compV[, -ind]
+    #     namesComp = namesComp[-ind]
+    # }
     if (length(size(sys$comp)) == 1){
         if (is.ts(sys$y)){
             sys$comp = ts(matrix(sys$comp, n + sys$h, 1), start = start(sys$y, frequency = frequency(sys$y)), frequency = frequency(sys$y))
@@ -117,8 +121,13 @@ UCcomponents= function(sys){
             sys$compV = matrix(sys$compV, n + sys$h, 1)
         }
     }
-    colnames(sys$comp) = namesComp
-    colnames(sys$compV) = namesComp
+    # colnames(sys$comp) = namesComp
+    # colnames(sys$compV) = namesComp
+    # if (substr(output$compNames, nchar(output$compNames), nchar(output$compNames)) == "/")
+    #     output$compNames = substr(output$compNames, 1, nchar(output$compNames) - 1)
+    names = strsplit(output$compNames, "/")
+    colnames(sys$comp) = names[[1]]
+    colnames(sys$compV) = names[[1]]
     
     return(sys)
 }
